from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from interfaces.BloggerAPI import IBloggerAPI
from Products.CMFCore.utils import getToolByName
from DateTime import DateTime
import xmlrpclib
import calendar

XMLRPCTRUE = xmlrpclib.Boolean(1)
XMLRPCFALSE = xmlrpclib.Boolean(0)

authOneMethods = ['blogger/getTemplate','blogger/setTemplate','blogger/deletePost'
                  ,'blogger/editPost','blogger/newPost','blogger/getRecentPosts']
def genericBloggerAuthOne(arg_tuple):
    return arg_tuple[2],arg_tuple[3],arg_tuple

authTwoMethods = ['blogger/getUsersBlogs','blogger/getUserInfo']
def genericBloggerAuthTwo(arg_tuple):
    return arg_tuple[1],arg_tuple[2],arg_tuple

def findRPCAuth(parent):
    while hasattr(parent,'aq_parent'):
        RPCAuths = parent.objectValues('RPC Auth')
        for RPCAuth in RPCAuths:
            return RPCAuth
        parent = parent.aq_parent
    return None

def addBloggerAPI(self, id, title=''):
    o=BloggerAPI(id, title)
    self._setOb(id, o)
    o=self._getOb(id, None)
    return o

class BloggerAPI(SimpleItem):
    meta_type='BloggerAPI'
    __implements__=IBloggerAPI
    security=ClassSecurityInfo()
    
    def __init__(self, id, title=''):
        self.id=id
        self.title=title

    def manage_afterAdd(self,item,container):
        RPCAuth = findRPCAuth(container)
        if RPCAuth:
            RPCAuth.addAuthProvider(authOneMethods, genericBloggerAuthOne)
            RPCAuth.addAuthProvider(authTwoMethods, genericBloggerAuthTwo)
        else:
            raise "No RPCAuth Instance","Please make sure RPCAuth is install correctly"

        
    security.declarePublic('getTemplate')
    def getTemplate(self, appkey, blogid, username, password, templateType):
        """ ?? """
        raise NotImplementedError

    security.declarePublic('setTemplate')
    def setTemplate(self, appkey, blogid, username, password, template, templateType):
        """ ?? """ 
        raise NotImplementedError

    security.declarePublic('deletePost')
    def deletePost(self, appkey, postid,username,password, publish):
        """ removes a blogger posting """
        catalog=getToolByName(self, 'portal_catalog')
        objs=catalog(id=postid)
        obj=None
        if objs:
            obj=objs[0].getObject()
        else:
            raise 'KeyError', 'no post %s' % postid
        if obj is not None:
            obj.aq_parent.manage_delObjects(postid)
	    return XMLRPCTRUE
        return XMLRPCFALSE          

    security.declarePublic('editPost')
    def editPost(self, appkey, postid, username, password, content, publish, post_format='stx'):
        """ modify a blogger posting """
        home=getToolByName(self, 'portal_membership').getHomeFolder()
        catalog=getToolByName(self, 'portal_catalog')
        objs=catalog(id=postid)
        if len(objs) > 0:
            obj=objs[0].getObject()
            obj.setDescription(content)
            format = obj.format
            obj.edit(format,content)
            return XMLRPCTRUE
        return XMLRPCFALSE

    security.declarePublic('newPost')
    def newPost(self, appkey, blogid, username, password, content, publish=None, post_format=None):
        """ create a new blog entry 
        This makes a Document with a unique id if there is no container path 
        the function uses the members home directory.  Make sure to set home
        directory creation on if your members are imported from a external user
        folder.  The first 6 options are required for blogger compatiblity, but the
        functionality can be extended in Zope by using default values for agruements
        like post_format.
        """
        uid=self.getNewBlogEntryId()
        portalObject=getToolByName(self, 'portal_url').getPortalObject()
        if blogid.lower() == 'home':
            container=getToolByName(self, 'portal_membership').getHomeFolder()
            if not container:
                container = portalObject
        else:
            container=portalObject.restrictedTraverse(blogid)
        if not container: raise AttributeError,"blog id %s is not a valid id" % blogid

	postTitle=content[:content.find('\n')][:30] # first line is title up to 30 chars
        container.invokeFactory('Blog Entry'
                          , uid
                          , title=postTitle
                          , description=content
                          , text_format=post_format
                          , text=content
                          )
        item=container._getOb(uid, None)
        if publish:
            wf_tool=getToolByName(self, 'portal_workflow')
            wf_tool.doActionFor(item, 'publish')       #XXX maybe this should be handled by a blog_workflow
	if item is not None:
            return uid 
        raise 'newPost Error', 'The item was not created.'

    security.declarePublic('getUserInfo') #protected 
    def getUserInfo(self, appkey, username, password):
        """ returns user information as a dictionary """
        membership=getToolByName(self, 'portal_membership')
        info={}
        member=membership.getAuthenticatedMember()
        info['name']=getattr(member, 'blogName', 'no name')
        info['email']=getattr(member, 'email', 'no email')
        info['userid']=getattr(member, 'username', 'no id?')
        info['firstname']=getattr(member, 'firstname', 'no firstname')
        info['lastname']=getattr(member, 'lastname', 'no lastname')
        info['url']=getattr(member, 'blogLocation', 'no url')
        return info

    security.declarePublic('getRecentPosts')
    def getRecentPosts(self, appkey, blogid, username, password, numberOfPosts=None):
        """ gets a list of posts by a user. The number of posts is unlimited"""
        today=DateTime()
        year, month=today.year(), today.month()
        mrange=calendar.monthrange(year,month)
	results=[]
	if mrange[0]<1: 
            mrange=(1,mrange[1])
        firstday=DateTime('%s-%s-%s 00:00:00' % (year,month,mrange[0]))
        for blog in self.aq_parent.getPosts(blogid): 
	    obj=blog.getObject()
	    results.append( { 'dateCreated':blog.created
			    , 'userid':blog.Creator
                            , 'postid':blog.id
                            , 'content':obj.EditableBody() } )
        if numberOfPosts is not None:
            return results[:int(numberOfPosts)]
        return results

    security.declarePublic('getUsersBlogs')
    def getUsersBlogs(self, appkey, username, password):
        """ script returns basic information about a users blogs:
        'url' is set to the absolute url of the blogLocation
        'blogid' is set to the users blogLocation (a relative url)
        'blogname' is set to the blogLocation folder's title or id """
        weblogs = self.aq_parent.getWeblogContainers()
        url_tool = portalObject=getToolByName(self, 'portal_url')
        info = [{'url':weblogs[0].absolute_url(),'blogid':'Home','blogname':weblogs[0].title_or_id()}]
        for log in weblogs[1:]:
            info.append({'url':log.absolute_url(),
                         'blogid':url_tool.getRelativeContentURL(log),
                         'blogname':log.title_or_id()})
        return info
InitializeClass(BloggerAPI)
