from Products.CMFCore.Expression import Expression, createExprContext
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import _checkPermission, _getAuthenticatedUser, limitGrantedRoles
from Products.CMFCore.utils import getToolByName, _dtmldir
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.PageTemplate import PageTemplate
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Acquisition import aq_base, aq_inner, aq_parent
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.CMFCore.TypesTool import FactoryTypeInformation as FTI
import string

from DateTime import DateTime
from Products.CMFCore.PortalFolder import PortalFolder
import BloggerAPI

import xmlrpclib
XMLRPCTRUE=xmlrpclib.Boolean(1)
XMLRPCFALSE=xmlrpclib.Boolean(0)
__version__='0.9.9'

def genericBloggerAuthOne(arg_tuple):
    return arg_tuple[2],arg_tuple[3],arg_tuple

def genericBloggerAuthTwo(arg_tuple):
    return arg_tuple[1],arg_tuple[2],arg_tuple


from Products.CMFCore.DirectoryView import addDirectoryViews
def setupWeblogs(WeblogTool):
    """ installs the CMFWeblog """
    typesTool=getToolByName(WeblogTool, 'portal_types')
    memberdataTool=getToolByName(WeblogTool, 'portal_memberdata')
    skinsTool=getToolByName(WeblogTool, 'portal_skins')
    if 'Blog Entry' not in typesTool.objectIds():
        try:
            typesTool.manage_addTypeInformation(id='Blog Entry', typeinfo_name='CMFDefault: Document')
        except:
            typesTool.manage_addTypeInformation(FTI.meta_type, id='Blog Entry', typeinfo_name='CMFDefault: Document')
            blogType=typesTool.getTypeInfo('Blog Entry')
            blogType.manage_changeProperties(content_meta_type='Blog Entry')
    if not memberdataTool.hasProperty('blogLocations'):
        memberdataTool.manage_addProperty(id='blogLocations',value='Home',type='lines')

    #register CMFWeblog as a ActionProvider / using TTW method because of problem with ActionsTool
    getToolByName(WeblogTool,'portal_actions').manage_aproviders(add_provider=1,apname='portal_weblog')

    #setup skins
    try:
         addDirectoryViews(skinsTool,'skins',globals())
    except:
        pass
    skin_names = skinsTool.getSkinSelections()
    for skin_name in skin_names:
        path = string.split(skinsTool.getSkinPath(skin_name).replace(' ',''),',')
        try:
            if not 'weblog_views' in path: path.insert(path.index( 'custom')+1, 'weblog_views' )
        except ValueError:
            path.append( 'weblog_views' )
        path = string.join(path,', ')
        skinsTool.addSkinSelection(skin_name,path)

    
class WeblogTool (UniqueObject, PortalFolder, ActionProviderBase):
    """ CMF Weblogger """
    id = 'portal_weblog'
    meta_type= 'CMF Weblog Tool'
    _actions = [ActionInformation(id='add to weblogs'
                                , title='add to weblogs'
                                , action=Expression(
                text='python: "%s/addWeblog?RPath=%s" % (object.portal_weblog.absolute_url(), object.portal_url.getRelativeContentURL(object))')                                
                , condition=Expression(text='python: not object.meta_type.lower().find("folder")==-1')
                , permissions=('Add portal content',)
                                , category='folder'
                                , visible=1
                                 ),]
    security = ClassSecurityInfo()

    _properties=({'id':'blogPrefix','type':'string'},)

    def __init__(self, id='portal_weblog'):
        self.id=id
        self.blogPrefix='blog_'

    def manage_afterAdd(self,item,container):
        self._setObject('blogger',BloggerAPI.BloggerAPI('blogger','Handles Blogger API Functions'))
        setupWeblogs(self)

    def addWeblog(self,RPath,REQUEST=None):
        """adds folder to users blogLocations"""
        membership=getToolByName(self, 'portal_membership')
        urlTool = getToolByName(self, 'portal_url')
        member=membership.getAuthenticatedMember()
        myBlogLocations = getattr(member,'blogLocations',[])        
        if not RPath in myBlogLocations: 
            myBlogLocations.append(RPath)
            member.setProperties(blogLocations=myBlogLocations)
        if REQUEST:
            REQUEST.RESPONSE.setStatus('MovedTemporarily')
            REQUEST.RESPONSE.setHeader('location',urlTool()+"/"+RPath)

    #utility function
    def getNewBlogEntryId(self):
        prefix=getattr(self,'blogPrefix','blog_')
        ep=str(DateTime().millis())
        return prefix+ep[len(ep)-5:len(ep)]
 
    #simple versions of some blogger api functions for use within Zope
    def getPosts(self,blogid):
        urltool=getToolByName(self, 'portal_url')
        member=getToolByName(self, 'portal_membership').getAuthenticatedMember()
        catalog=getToolByName(self, 'portal_catalog')
        if blogid=='home':
           homeFolder = container=getToolByName(self, 'portal_membership').getHomeFolder()
           if homeFolder: blogid=urltool.getRelativeContentURL(homeFolder)
           else: raise "PortalMemberError","User has no homefolder"
        elif blogid=='all': pass
        return catalog( path={'query':blogid,'level':-1} 
                           , Creator=member.id, Type='Blog Entry'
			   , sort_on='created', sort_order='reverse' )

    def getWeblogContainers(self):
        """
        returns a list of container objects. the first object is the users homedir
        """
        weblogs = []
        membership=getToolByName(self, 'portal_membership')
        member=membership.getAuthenticatedMember()
        portalObject=getToolByName(self, 'portal_url').getPortalObject()
        home=membership.getHomeFolder()
        if home:
            weblogs.append(home)
        else:
            weblogs.append(portalObject)
        blogLocations=getattr(member, 'blogLocations', None)
        if blogLocations:
            for location in blogLocations:
                if location.lower() == 'home':continue
                locObject = portalObject.restrictedTraverse(location)
                if locObject: weblogs.append(locObject)
        return weblogs
       
      
InitializeClass(WeblogTool)
